@@ -13,7 +13,7 @@ use Masterkey::Admin::Config;
 use Masterkey::Admin::Torus;
 
 our @ISA = qw();
-our $VERSION = '4.3';
+our $VERSION = '4.4';
 
 sub new {
     my $class = shift();
@@ -164,27 +164,78 @@ sub new_with_session {
 	}
     }
 
+    my %criteria;
     if ($uid) {
-	eval {
-	    $user = $admin->authTorus()->user1(id => $uid);
-	}; if ($@) {
-	    if (ref($@) eq "HTML::Mason::Exception" && $@ =~ 404) {
-		# Assume the record has been deleted
-		goto DONE;
-	    } else {
-		die $@;
-	    }
+	%criteria = (id => $uid);
+    } else {
+	# We've not been able to log in; but if we're logged in as the
+	# consortium, then we can just act as the user anyway.
+	#
+	# ### I'm unhappy about hardwired knowledge of domains here.
+	#
+	if (loggedInAtDomain($r, 'wizard')) {
+	    warn "wizard finding user by realm '$identityId'";
+	    %criteria = (identityId => $identityId);
 	}
-	die "Invalid user-ID '$uid'" if !defined $user;
+    }
 
-	# Special case: consortium-level administrator is always super
-	$user->setField(superuser => 1) if $domain eq "wizard";
+    if (!%criteria) {
+	goto DONE;
+    }
+
+    eval {
+	$user = $admin->authTorus()->user1(%criteria);
+    }; if ($@) {
+	if (ref($@) eq "HTML::Mason::Exception" && $@ =~ 404) {
+	    # Assume the record has been deleted
+	    goto DONE;
+	} else {
+	    die $@;
+	}
+    }
+
+    if (!defined $user) {
+	if ($uid) {
+	    die "Invalid user-ID '$uid'";
+	} else {
+	    die "Invalid user-realm '$identityId'";
+	}
+    }
+
+    # Special case: consortium-level administrator is always super
+    $user->setField(superuser => 1) if $domain eq "wizard";
+
+    if (!$uid) {
+	# This session was created from the consortium level: patch it up.
+	warn "patching up wizard-induced session";
+	$session->update(user_id => $user->field('id'));
     }
 
   DONE:
     return ($admin, $session, $user);
 }
 
+
+sub loggedInAtDomain {
+    my($r, $domain) = @_;
+
+    my @links;
+    my $cookies = Apache2::Cookie->fetch($r);
+
+    foreach my $key (sort keys %$cookies) {
+	if ($key =~ /^session-$domain-/) {
+	    my $cookie = $cookies->{$key};
+	    my $sessid = $cookie->value();
+	    push @links, [ $sessid ];
+	}
+    }
+
+    warn "got ", scalar(@links), " sessions in $domain domain";
+    @links = Masterkey::Admin::Session::filter_links(@links);
+    warn "retained ", scalar(@links), " logged-in sessions in $domain domain";
+    return @links ? 1 : 0;
+}
+
 sub siteConf {
     my $this = shift();
     my($key) = @_;
