@@ -56,7 +56,7 @@ sub zurlAndAuth {
 	my $torusRealm = $admin2->conf("torusRealm");
 	my $torus2 = $admin2->torus();
 	my $rs = $torus2->records($torusRealm, undef, $adminUser);
-	die "no identity record to test this target with" if $rs->count() == 0;
+	die "no user-access record to test this target with" if $rs->count() == 0;
 	warn "multiple identity records; using first" if $rs->count() > 1;
 	my $record2 = $rs->record(0);
 	$auth{user} = $record2->field("userName");
